#include "elemswitcht.h"
#include "globalt.h"
#include "globmatt.h"

/**
   function computes conductivity %matrix of one element
   
   @param eid - element id
   @param lcid - load case id
   @param km - conductivity %matrix of one element

*/
void conductmat (long eid,long lcid,matrix &km)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);
  
  switch (te){
  case barlint:{
    Lbt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case barlintax:{
    Lbat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case barquadt:{
    Qbt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case barquadtax:{
    Qbat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case trlint:{
    Ltt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case trlaxisym:{
    Ltat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadlint:{
    Lqt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadquadt:{
    Qqt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadquadtax:{
    Qqat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadlaxisym:{
    Lqat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case ifacequadel:{
    Ifcquadt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case lineartett:{
    Ltett->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case linearhext:{
    Lht->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadratichext:{
    Qht->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}

/**
   function computes capacity %matrix of one element

   @param eid - element id
   @param lcid - load case id
   @param km - capacity %matrix of one element

*/
void capacmat (long eid,long lcid,matrix &cm)
{
  elemtypet te;

  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->res_capacity_matrix (eid,cm);
    break;
  }
  case barlintax:{
    Lbat->res_capacity_matrix (eid,cm);
    break;
  }
  case barquadt:{
    Qbt->res_capacity_matrix (eid,cm);
    break;
  }
  case barquadtax:{
    Qbat->res_capacity_matrix (eid,cm);
    break;
  }
  case trlint:{
    Ltt->res_capacity_matrix (eid,cm);
    break;
  }
  case trlaxisym:{
    Ltat->res_capacity_matrix (eid,cm);
    break;
  }
  case quadlint:{
    Lqt->res_capacity_matrix (eid,cm);
    break;
  }
  case quadquadt:{
    Qqt->res_capacity_matrix (eid,cm);
    break;
  }
  case quadquadtax:{
    Qqat->res_capacity_matrix (eid,cm);
    break;
  }
  case quadlaxisym:{
    Lqat->res_capacity_matrix (eid,cm);
    break;
  }
  case ifacequadel:{
    Ifcquadt->res_capacity_matrix (eid,cm);
    break;
  }
  case lineartett:{
    Ltett->res_capacity_matrix (eid,cm);
    break;
  }
  case linearhext:{
    Lht->res_capacity_matrix (eid,cm);
    break;
  }
  case quadratichext:{
    Qht->res_capacity_matrix (eid,cm);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function approximates nodal values to integration points
   
   @param eid - element id
   
   JK
*/
void intpointvalues (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointval (eid);
    break;
  }
  case barlintax:{
    Lbat->intpointval (eid);
    break;
  }
  case barquadt:{
    Qbt->intpointval (eid);
    break;
  }
  case barquadtax:{
    Qbat->intpointval (eid);
    break;
  }
  case trlint:{
    Ltt->intpointval (eid);
    break;
  }
  case trlaxisym:{
    Ltat->intpointval (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointval (eid);
    break;
  }
  case quadquadt:{
    Qqt->intpointval (eid);
    break;
  }
  case quadquadtax:{
    Qqat->intpointval (eid);
    break;
  }
  case quadlaxisym:{
    Lqat->intpointval (eid);
    break;
  }
  case ifacequadel:{
    //Ifcquadt->intpointval (eid);
    break;
  }
  case lineartett:{
    Ltett->intpointval (eid);
    break;
  }
  case linearhext:{
    Lht->intpointval (eid);
    break;
  }
  case quadratichext:{
    Qht->intpointval (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}



/**
   function approximates nodal values of actual unknown to integration points of PUC
   
   @param eid - element id
   
   TKr, 08/09/2010 
*/
void intpointvalues_puc (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case trlint:{
    Ltt->intpointval_puc (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointval_puc (eid);
    break;
  }
  case lineartett:{
    Ltett->intpointval_puc (eid);
    break;
  }
  case linearhext:{
    Lht->intpointval_puc (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes gradients on integration points
   
   @param eid - element id
   
   JK, 10.7.2008
*/
void intpointgradients (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointgrad (eid);
    break;
  }
  case barlintax:{
    Lbat->intpointgrad (eid);
    break;
  }
  case barquadt:{
    Qbt->intpointgrad (eid);
    break;
  }
  case barquadtax:{
    Qbat->intpointgrad (eid);
    break;
  }
  case trlint:{
    Ltt->intpointgrad (eid);
    break;
  }
  case trlaxisym:{
    Ltat->intpointgrad (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointgrad (eid);
    break;
  }
  case quadquadt:{
    Qqt->intpointgrad (eid);
    break;
  }
  case quadquadtax:{
    Qqat->intpointgrad (eid);
    break;
  }
  case quadlaxisym:{
    Lqat->intpointgrad (eid);
    break;
  }
  case ifacequadel:{
    //Ifcquadt->intpointgrad (eid);
    break;
  }
  case lineartett:{
    Ltett->intpointgrad (eid);
    break;
  }
  case linearhext:{
    Lht->intpointgrad (eid);
    break;
  }
  case quadratichext:{
    Qht->intpointgrad (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes fluxes on integration points
   
   @param eid - element id
   
   TKr, 01/02/2010
*/
void intpointfluxes (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointflux (eid);
    break;
  }
  case barlintax:{
    //Lbat->intpointflux (eid);
    break;
  }
  case barquadt:{
    //Qbt->intpointflux (eid);
    break;
  }
  case barquadtax:{
    //Qbat->intpointflux (eid);
    break;
  }
  case trlint:{
    //Ltt->intpointflux (eid);
    break;
  }
  case trlaxisym:{
    //Ltat->intpointflux (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointflux (eid);
    break;
  }
  case quadquadt:{
    Qqt->intpointflux (eid);
    break;
  }
  case quadquadtax:{
    //Qqat->intpointflux (eid);
    break;
  }
  case quadlaxisym:{
    //Lqat->intpointflux (eid);
    break;
  }
  case ifacequadel:{
    //Ifcquadt->intpointflux (eid);
    break;
  }
  case lineartett:{
    //Ltett->intpointflux (eid);
    break;
  }
  case linearhext:{
    Lht->intpointflux (eid);
    break;
  }
  case quadratichext:{
    //Qht->intpointflux (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function computes values of array other at integration points
   
   @param lcid - load case id, id of the medium
   @param eid - element id
   @param fl - %vector containing flux
   
   JK, 12. 8. 2014
*/
void averageflux (long lcid,long eid,vector &fl)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    //Lbt->intpointother (eid);
    break;
  }
  case barlintax:{
    //Lbat->intpointother (eid);
    break;
  }
  case barquadt:{
    //Qbt->intpointother (eid);
    break;
  }
  case barquadtax:{
    //Qbat->intpointother (eid);
    break;
  }
  case trlint:{
    //Ltt->intpointother (eid);
    break;
  }
  case trlaxisym:{
    //Ltat->intpointother (eid);
    break;
  }
  case quadlint:{
    //Lqt->intpointother (eid);
    break;
  }
  case quadquadt:{
    //Qqt->intpointother (eid);
    break;
  }
  case quadquadtax:{
    //Qqat->intpointother (eid);
    break;
  }
  case quadlaxisym:{
    //Lqat->intpointother (eid);
    break;
  }
  case ifacequadel:{
    //Ifcquadt->intpointother (eid);
    break;
  }
  case lineartett:{
    //Ltett->intpointother (eid);
    break;
  }
  case linearhext:{
    Lht->average_flux (lcid,eid,fl);
    break;
  }
  case quadratichext:{
    //Qht->intpointother (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes values of array other at integration points
   
   @param eid - element id
   
   JK
*/
void intpointothers (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointother (eid);
    break;
  }
  case barlintax:{
    Lbat->intpointother (eid);
    break;
  }
  case barquadt:{
    Qbt->intpointother (eid);
    break;
  }
  case barquadtax:{
    Qbat->intpointother (eid);
    break;
  }
  case trlint:{
    Ltt->intpointother (eid);
    break;
  }
  case trlaxisym:{
    Ltat->intpointother (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointother (eid);
    break;
  }
  case quadquadt:{
    Qqt->intpointother (eid);
    break;
  }
  case quadquadtax:{
    Qqat->intpointother (eid);
    break;
  }
  case quadlaxisym:{
    Lqat->intpointother (eid);
    break;
  }
  case ifacequadel:{
    //Ifcquadt->intpointother (eid);
    break;
  }
  case lineartett:{
    Ltett->intpointother (eid);
    break;
  }
  case linearhext:{
    Lht->intpointother (eid);
    break;
  }
  case quadratichext:{
    Qht->intpointother (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function computes values of array eqother at integration points
   
   @param eid - element id
   
   TKr, 01/02/2010
*/
void intpointeqothers (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    //Lbt->intpointeqother (eid);
    break;
  }
  case barlintax:{
    //Lbat->intpointeqother (eid);
    break;
  }
  case barquadt:{
    //Qbt->intpointeqother (eid);
    break;
  }
  case barquadtax:{
    //Qbat->intpointeqother (eid);
    break;
  }
  case trlint:{
    //Ltt->intpointeqother (eid);
    break;
  }
  case trlaxisym:{
    //Ltat->intpointeqother (eid);
    break;
  }
  case quadlint:{
    //Lqt->intpointeqother (eid);
    break;
  }
  case quadquadt:{
    //Qqt->intpointeqother (eid);
    break;
  }
  case quadquadtax:{
    //Qqat->intpointeqother (eid);
    break;
  }
  case quadlaxisym:{
    //Lqat->intpointeqother (eid);
    break;
  }
  case ifacequadel:{
    //Ifcquadt->intpointeqother (eid);
    break;
  }
  case lineartett:{
    //Ltett->intpointeqother (eid);
    break;
  }
  case linearhext:{
    //Lht->intpointeqother (eid);
    break;
  }
  case quadratichext:{
    //Qht->intpointeqother (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes contributions from source of quantity
   
   @param lcid - load case id
   @param eid - element id
   @param nodval - array of source nodal values
   @param lv - local %vector
   
   JK, 25.6.2005
*/
void source_vector (long lcid,long eid,vector &nodval,vector &lv)
{
  elemtypet te;
  
  //  type of element
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case barlintax:{
    Lbat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case trlint:{
    Ltt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case trlaxisym:{
    Ltat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadlint:{
    Lqt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadlaxisym:{
    Lqat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadquadt:{
    Qqt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadquadtax:{
    Qqat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case ifacequadel:{
    break;
  }
  case lineartett:{
    Ltett->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case linearhext:{
    Lht->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadratichext:{
    Qht->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}



/**
   The function computes fluxes at integration points.   
   All components at all integration points are evaluated.
   
   @param lcid - load case id
   
   @return The function does not return anything.
   
   Created by TKr, 01/02/2010
*/
void compute_ipfluxes ()
{
  long i,ne;
  
  //  number of elements
  ne=Tt->ne;
  
  for (i=0;i<ne;i++){
    if (Gtt->leso[i]==1){
      //  only elements switched on are taken into account
      intpointfluxes (i);
    }
  }
}

/**
   The function computes average flux
   All components in all integration points are taken into account.
   The function is used in homogenization/tiling.

   @param lcid - load case id
   
   @return The function does not return anything.
   
   JK, 12. 8. 2014
*/
void compute_average_fluxes (long lcid, vector &fl)
{
  long i,ne;
  
  //  number of elements
  ne=Tt->ne;
  
  nullv (fl.a,fl.n);

  for (i=0;i<ne;i++){
    if (Gtt->leso[i]==1){
      //  only elements switched on are taken into account
      averageflux (lcid,i,fl);
    }
  }
}


/**
   The function computes gradients at integration points.   
   All components at all integration points are evaluated.
   
   @param lcid - load case id
   
   @return The function does not return anything.
   
   Created by LS, 31.8.2012
*/
void compute_ipgrads ()
{
  // loop over all elements
  for (long i=0; i<Gtt->ne; i++)
    if (Gtt->leso[i])
      //  only elements switched on are taken into account
      intpointgradients (i);
  
}


/**
  The function computes other values at integration points.   
  All components at all integration points are evaluated.
   
  @param lcid - load case id
   
  @return The function does not return anything.

  Created by TKr, 01/02/2010
*/
void compute_ipotherst ()
{
  long i,ne;
  
  //  number of elements
  ne=Tt->ne;

  for (i=0;i<ne;i++){
    if (Gtt->leso[i]==1){
      //  only elements switched on are taken into account
      intpointothers (i);
    }
  }

}


/**
  The function computes eqother values at integration points.   
  All components at all integration points are evaluated.
   
  @param lcid - load case id
   
  @return The function does not return anything.

  Created by TKr, 01/02/2010
*/
void compute_ipeqotherst ()
{
  long i,ne;
  
  //  number of elements
  ne=Tt->ne;

  for (i=0;i<ne;i++){
    if (Gtt->leso[i]==1){
      //  only elements switched on are taken into account
      intpointeqothers (i);
    }
  }

}


/**
  Function computes gradients at element nodes.
  Nodal gradients are averaged. In case of bar elements 
  be carefull if averaging is reasonable.

  @param lcid - load case id
  
  @return The function does not return anything.
 
  TKr, 01/02/2010
*/
void compute_nodegrads ()
{
  long i;
  elemtypet te;
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].nullgrad();
  }
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      te = Tt->give_elem_type (i);
      
      switch (te){
      case barlint:{
	Lbt->nod_grads_ip (i);
	break;
      }
      case barlintax:{
	//Lbat->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case barquadt:{
	//Qbt->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case barquadtax:{
	//Qbat->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case quadlint:{
	Lqt->nod_grads_ip (i);
	break;
      }
      case trlint:{
	//Ltt->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case trlaxisym:{
	//Ltat->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case quadquadt:{
	//Qqt->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case quadquadtax:{
	//Qqat->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case quadlaxisym:{
	//Lqat->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case ifacequadel:{
	//Ifcquadt->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case lineartett:{
	//Ltett->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case linearhext:{
	//Lht->nod_grads_ip (lcid,i,0,0);
	break;
      }
      case quadratichext:{
	//Qht->nod_grads_ip (lcid,i,0,0);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
      }
      }
    }
  }
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].grad_averageval ();
  }
}


/**
  Function computes fluxes at element nodes.
  Nodal fluxes are averaged. In case of bar elements 
  be carefull if averaging is reasonable.

  @param lcid - load case id
  
  @return The function does not return anything.
 
  TKr, 01/02/2010
*/
void compute_nodefluxes ()
{
  long i;
  elemtypet te;
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].nullflux();
  }
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      te = Tt->give_elem_type (i);
      
      switch (te){
      case barlint:{
	Lbt->nod_fluxes_ip (i);
	break;
      }
      case barlintax:{
	//Lbat->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case barquadt:{
	//Qbt->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case barquadtax:{
	//Qbat->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case quadlint:{
	Lqt->nod_fluxes_ip (i);
	break;
      }
      case trlint:{
	//Ltt->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case trlaxisym:{
	//Ltat->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case quadquadt:{
	//Qqt->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case quadquadtax:{
	//Qqat->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case quadlaxisym:{
	//Lqat->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case ifacequadel:{
	//Ifcquadt->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case lineartett:{
	//Ltett->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case linearhext:{
	//Lht->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      case quadratichext:{
	//Qht->nod_fluxes_ip (lcid,i,0,0);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
      }
      }
    }
  }
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].flux_averageval ();
  }
}


/**
  Function computes other values at element nodes.
  Nodal other values are averaged. In case of bar elements 
  be carefull if averaging is reasonable.

  @param lcid - load case id
  
  @return The function does not return anything.
 
  TKr, 01/02/2010
*/
void compute_nodeotherst ()
{
  long i;
  elemtypet te;
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].nullother();
  }
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      te = Tt->give_elem_type (i);
      
      switch (te){
      case barlint:{
	//Lbt->nod_others_ip (lcid,i,0,0);
	break;
      }
      case barlintax:{
	//Lbat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case barquadt:{
	//Qbt->nod_others_ip (lcid,i,0,0);
	break;
      }
      case barquadtax:{
	//Qbat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadlint:{
	//Lqt->nod_others_ip (lcid,i,0,0);
	break;
      }
      case trlint:{
	//Ltt->nod_others_ip (lcid,i,0,0);
	break;
      }
      case trlaxisym:{
	//Ltat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadquadt:{
	//Qqt->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadquadtax:{
	//Qqat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadlaxisym:{
	//Lqat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case ifacequadel:{
	//Ifcquadt->nod_others_ip (lcid,i,0,0);
	break;
      }
      case lineartett:{
	//Ltett->nod_others_ip (lcid,i,0,0);
	break;
      }
      case linearhext:{
	//Lht->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadratichext:{
	//Qht->nod_others_ip (lcid,i,0,0);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
      }
      }
    }
  }
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].other_averageval ();
  }
}



/**
  Function computes other values directly at element nodes.
  Nodal other values are averaged. In case of bar elements 
  be carefull if averaging is reasonable.

  @param lcid - load case id
  
  @return The function does not return anything.
 
  TKr, 01/02/2010
*/
void compute_nodeotherst_comp ()
{
  long i;
  elemtypet te;
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].nullother();
  }
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      te = Tt->give_elem_type (i);
      
      switch (te){
      case barlint:{
	Lbt->nod_others_comp (0,i,0,0);
	break;
      }
      case barlintax:{
	//Lbat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case barquadt:{
	Qbt->nod_others_comp (0,i,0,0);
	break;
      }
      case barquadtax:{
	//Qbat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadlint:{
	Lqt->nod_others_comp (0,i,0,0);
	break;
      }
      case trlint:{
	//Ltt->nod_others_ip (lcid,i,0,0);
	break;
      }
      case trlaxisym:{
	//Ltat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadquadt:{
	Qqt->nod_others_comp (0,i,0,0);
	break;
      }
      case quadquadtax:{
	//Qqat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case quadlaxisym:{
	//Lqat->nod_others_ip (lcid,i,0,0);
	break;
      }
      case ifacequadel:{
	break;
      }
      case lineartett:{
	//Ltett->nod_others_ip (lcid,i,0,0);
	break;
      }
      case linearhext:{
	Lht->nod_others_comp (0,i,0,0);
	break;
      }
      case quadratichext:{
	//Qht->nod_others_ip (lcid,i,0,0);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
      }
      }
    }
  }
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].other_averageval ();
  }
}


/**
  Function computes eqother values at element nodes.
  Nodal eqother values are averaged. In case of bar elements 
  be carefull if averaging is reasonable.

  @param lcid - load case id
  
  @return The function does not return anything.
 
  TKr, 01/02/2010
*/
void compute_nodeeqotherst ()
{
  long i;
  elemtypet te;
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].nulleqother();
  }
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      te = Tt->give_elem_type (i);
      
      switch (te){
      case barlint:{
	//Lbt->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case barlintax:{
	//Lbat->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case barquadt:{
	//Qbt->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case barquadtax:{
	//Qbat->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case quadlint:{
	Lqt->nod_eqother_ip (i);
	break;
      }
      case trlint:{
	//Ltt->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case trlaxisym:{
	//Ltat->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case quadquadt:{
	//Qqt->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case quadquadtax:{
	//Qqat->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case quadlaxisym:{
	//Lqat->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case ifacequadel:{
	break;
      }
      case lineartett:{
	//Ltett->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case linearhext:{
	//Lht->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      case quadratichext:{
	//Qht->nod_eqothers_ip (lcid,i,0,0);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
      }
      }
    }
  }
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].eqother_averageval ();
  }
}

/**
   function assembles %vector of nodal internal fluxes

   @param lcid - load case id
   @param intflux - array containing internal nodal fluxes
   
*/
void internal_fluxes (double *intflux,long n)
{
  long i,ne,ndofe,*cn;
  elemtypet te;
  vector lif;
  
  ne = Tt->ne;
  nullv (intflux,n);

  for (i=0;i<ne;i++){
    if (Gtt->leso[i]==1){

      te = Tt->give_elem_type (i);
      ndofe = Gtt->give_ndofe (i);
      allocv (ndofe,lif);
      fillv (0.0,lif);
      
      cn = new long [ndofe];
      Gtt->give_code_numbers (i,cn);
      
      switch (te){
      case barlint:{
	Lbt->res_internal_fluxes (i,lif);
	break;
      }
      case barlintax:{
	Lbat->res_internal_fluxes (i,lif);
	break;
      }
      case barquadt:{
	Qbt->res_internal_fluxes (i,lif);
	break;
      }
      case barquadtax:{
	Qbat->res_internal_fluxes (i,lif);
	break;
      }
      case trlint:{
	Ltt->res_internal_fluxes (i,lif);
	break;
      }
      case trlaxisym:{
	Ltat->res_internal_fluxes (i,lif);
	break;
      }
      case quadlint:{
	Lqt->res_internal_fluxes (i,lif);
	break;
      }
      case quadquadt:{
	Qqt->res_internal_fluxes (i,lif);
	break;
      }
      case quadquadtax:{
	Qqat->res_internal_fluxes (i,lif);
	break;
      }
      case quadlaxisym:{
	Lqat->res_internal_fluxes (i,lif);
	break;
      }
      case ifacequadel:{
	//Ifcquadt->res_internal_fluxes (i,lif);
	break;
      }
      case lineartett:{
	Ltett->res_internal_fluxes (i,lif);
	break;
      }
      case linearhext:{
	Lht->res_internal_fluxes (i,lif);
	break;
      }
      case quadratichext:{
	Qht->res_internal_fluxes (i,lif);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
	abort ();
      }
      }
      
      locglob (intflux,lif.a,cn,ndofe);
      destrv (lif);
      delete [] cn;
    }
  }

  trfel_bound_flux (0,intflux,n);

}

/**
   function computes nodal values on element defined by
   prescribed fluxes (Neumann boundary condition)
   
   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   @param i - index in the list of loaded elements
   
   JK, 22.11.2008
*/
void elem_neumann_vector (vector &lv,long lcid,long eid,long i)
{
  elemtypet te;
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case barlintax:{
    Lbat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case barquadt:{
    Qbt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case barquadtax:{
    Qbat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case trlint:{
    Ltt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case trlaxisym:{
    Ltat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadlint:{
    Lqt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadlaxisym:{
    Lqat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadt:{
    Qqt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadtax:{
    Qqat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case lineartett:{
    Ltett->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case linearhext:{
    Lht->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadratichext:{
    Qht->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes nodal values on element defined by
   prescribed transmission (Newton boundary condition)
   
   this function computes \int \kappa T_{ext}
   
   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   @param i - index in the list of loaded elements
   
   JK, 22.11.2008
*/
void elem_newton_vector (vector &lv,long lcid,long eid,long i)
{
  elemtypet te;
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case barlintax:{
    Lbat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case barquadt:{
    Qbt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case barquadtax:{
    Qbat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case trlint:{
    Ltt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case trlaxisym:{
    Ltat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadlint:{
    Lqt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadlaxisym:{
    Lqat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadt:{
    Qqt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadtax:{
    Qqat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case lineartett:{
    Ltett->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case linearhext:{
    Lht->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadratichext:{
    Qht->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function computes nodal values of boundary fluxes on element
   caused by transmission

   this function differs from the function elem_newton_vector
   this function computes fluxes on boundaries while the
   function elem_newton_vector computes terms for the right hand
   side %vector
   
   this function computes \int \kappa (T-T_{ext})

   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   @param i - index in the list of loaded elements
   
   JK, 22.11.2008
*/
void elem_transmission_flux (vector &lv,long lcid,long eid,long i)
{
  elemtypet te;
  
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case barlintax:{
    Lbat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case barquadt:{
    Qbt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case barquadtax:{
    Qbat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case trlint:{
    Ltt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case trlaxisym:{
    Ltat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadlint:{
    Lqt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadlaxisym:{
    Lqat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadquadt:{
    Qqt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadquadtax:{
    Qqat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case lineartett:{
    Ltett->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case linearhext:{
    Lht->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadratichext:{
    Qht->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function computes volume integral
   
   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   
   JK, 22.11.2008, corrected by TKr 10/12/2013
*/
void volume_rhs_vector (vector &lv,long lcid,long eid)
{
  elemtypet te;
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_volume_rhs_vector (lv,eid,lcid);
    break;
  }
  case barquadt:{
    Qbt->res_volume_rhs_vector (lv,eid,lcid);
    break;
  }
    /*
      case barlintax:{
      break;
      }
      case barquadtax:{
      break;
      }
      case trlaxisym:{
      break;
      }
    */
  case trlint:{
    break;
  }
    
  case quadlint:{
    Lqt->res_volume_rhs_vector (lv,eid,lcid);
    break;
  }
  case quadlaxisym:{
    Lqat->res_volume_rhs_vector (lv,eid,lcid);
    break;
  }
  case ifacequadel:{
       break;
  }
  case linearhext:{
    Lht->res_volume_rhs_vector (lv,eid,lcid);
    break;
  }
  case lineartett:{
    break;
  }
    
    /*     case quadquadtax:{
	   break;
	   }
	   
	   case quadratichext:{
	   break;
	   }
    */
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes l_t %matrix of one element
   
   @param eid - element id
   @param lcid - load case id
   @param lm - l %matrix of one element

*/
void lmat (long eid,long lcid,matrix &lm)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);
  
  switch (te){
  case trlint:{
    Ltt->res_l_matrix (eid,lcid,lm);
    break;
  }
  case quadlint:{
    Lqt->res_l_matrix (eid,lcid,lm);
    break;
  }
  case lineartett:{
    Ltett->res_l_matrix (eid,lcid,lm);
    break;
  }
  case linearhext:{
    Lht->res_l_matrix (eid,lcid,lm);
    break;
  }

  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}


/**
   function computes l_t %matrix of one element
   
   @param eid - element id
   @param lcid - load case id
   @param lm - l_t %matrix of one element

*/
void ltmat (long eid,long lcid,matrix &lm)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);
  
  switch (te){
  case trlint:{
    Ltt->res_l_t_matrix (eid,lcid,lm);
    break;
  }
  case quadlint:{
    Lqt->res_l_t_matrix (eid,lcid,lm);
    break;
  }
  case lineartett:{
    Ltett->res_l_t_matrix (eid,lcid,lm);
    break;
  }
  case linearhext:{
    Lht->res_l_t_matrix (eid,lcid,lm);
    break;
  }

  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}


/**
   function computes average D %matrix of one element
   
   @param eid - element id
   @param km - d %matrix of one element
   
*/
void averdmat (long eid,double &elemarea,matrix &km)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);
  
  switch (te){
  case trlint:{
    Ltt->averd_matrix (eid,km);
    elemarea = Ltt->elem_area(eid);
    break;
  }
  case quadlint:{
    Lqt->averd_matrix (eid,km);
    elemarea = Lqt->elem_area(eid);
    break;
  }
  case lineartett:{
    Ltett->averd_matrix (eid,km);
    elemarea = Ltett->elem_volume(eid);
    break;
  } 
  case linearhext:{
    Lht->averd_matrix (eid,km);
    elemarea = Lht->elem_volume(eid);
    break;
  }
    
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function computes average C %matrix of one element
   
   @param eid - element id
   @param cm - c %matrix of one element
   
*/
void avercmat (long eid,double &elemarea,matrix &cm)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);
  
  switch (te){
  case trlint:{
    Ltt->averc_matrix (eid,cm);
    elemarea = Ltt->elem_area(eid);
    break;
  }
  case quadlint:{
    Lqt->averc_matrix (eid,cm);
    elemarea = Lqt->elem_area(eid);
    break;
  }
  case lineartett:{
    Ltett->averc_matrix (eid,cm);
    elemarea = Ltett->elem_volume(eid);
    break;
  }
  case linearhext:{
    Lht->averc_matrix (eid,cm);
    elemarea = Lht->elem_volume(eid);
    break;
  }
    
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function returns area/volume of one element
   
   @param eid - element id
*/
double give_elemarea (long eid)
{
  elemtypet te;
  double elemarea;

  te=Tt->give_elem_type (eid);
  
  switch (te){
  case trlint:{
    elemarea = Ltt->elem_area(eid);
    break;
  }
  case quadlint:{
    elemarea = Lqt->elem_area(eid);
    break;
  }
  case lineartett:{
    elemarea = Ltett->elem_volume(eid);
    break;
  } 
  case linearhext:{
    elemarea = Lht->elem_volume(eid);
    break;
  }
    
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }

  return elemarea;
}



/**
   function selects components from the global level
   
   @param eid - element id
   @param counter - actual position in the array buff
   @param buff - array containing components
   
   JK, 25.3.2011
*/
void higher_to_lower_level_elem (long eid,long *counter,double *buff)
{
  elemtypet te;
  
  //  type of element
  te=Tt->give_elem_type (eid);
  
  switch (te){
  case trlint:{
    Ltt->higher_to_lower_level (eid,counter,buff);
    break;
  }
  case quadlint:{
    Lqt->higher_to_lower_level (eid,counter,buff);
    break;
  }
  case lineartett:{
    Ltett->higher_to_lower_level (eid,counter,buff);
    break;
  }
  case linearhext:{
    Lht->higher_to_lower_level (eid,counter,buff);
    break;
  }

  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}



/**
  The function interpolates nodal values into integration points.

  @param gv - array containing values at all nodes of the mesh
  @param ntq - type of non-transport quantity
  @param scale - scale coefficient
   
  @return The function does not return anything.
 
  12/06/2012 TKr according to JK
*/
void intpointvalt (double *gv, nontransquant ntq, double scale)
{
  long i,j,nne,nip,ipid;
  ivector nodes;
  vector nodval,ipval;
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      nne = Tt->give_nne (i);
      nip = Tt->give_tnip (i);
      
      reallocv (nne,nodes);
      reallocv (nne,nodval);
      reallocv (nip,ipval);
      
      //  nodes on element
      Tt->give_elemnodes (i,nodes);
      
      //  nodal values on element
      for (j=0;j<nne;j++){
	nodval[j]=gv[nodes[j]]*scale;
      }
      
      //  interpolation of nodal values to integration points
      elem_intpointvalt (i,nodval,ipval);
      
      //  number of the first integration point
      ipid=Tt->elements[i].ipp[0][0];
      
      for (j=0;j<nip;j++){
	Tm->storenontransq(ntq, ipid, ipval[j]);
	ipid++;
      }
    }
  }
}

/**
   function approximates nodal values to integration points

   @param eid - element id
   @param nodval - %vector of nodal values
   @param ipval - %vector of values at integration points (output)
   
   12/06/2012 TKr according to JK
*/
void elem_intpointvalt (long eid,vector &nodval,vector &ipval)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointval (eid,nodval,ipval);
    break;
  }
  case quadlint:{
    Lqt->intpointval (eid,nodval,ipval);
    break;
  }
  case quadlaxisym:{
    Lqat->intpointval (eid,nodval,ipval);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function approximates nontransport quantity from integration points to nodes
   
   @param gv - vector of nodal nontransport quantity
   @param mnt - global number of nodes
   @param nmq - type of transport (non-mechanical) quantity
 
  13/09/2012 TKr
  Modified 7.10.2013 TKo
*/
void give_transq_nodval (double *gv,long mnt,nonmechquant nmq)
{
  long i,j,nne;
  ivector nodes;
  vector nodval;
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      nne = Tt->give_nne (i);
      
      reallocv (nne,nodes);
      reallocv (nne,nodval);
      
      //  nodes on element
      Tt->give_elemnodes (i,nodes);
      
      //  nodal values on element
      elem_transq_nodval (i, nodval, nmq);
            
      for (j=0;j<nne;j++){
	gv[nodes[j]] = nodval[j];
      }
    }
  }
}



/**
  The function returns transport quantities from element nodes. The quantities 
  are copied from the closest integration points to element nodes.

  @param eid - element id
  @param nodval - %vector of nodal values
  @param nmq - type of transport (non-mechanical) quantity
   
  @return The function does not return anything.
 
  13/08/2012 TKr
  Modified 7.10.2013 TKo
*/
void elem_transq_nodval (long eid,vector &nodval,nonmechquant nmq)
{
  elemtypet te;

  te = Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->transq_nodval(eid, nodval, nmq);
    break;
  }
  case quadlint:{
    Lqt->transq_nodval(eid, nodval, nmq);
    break;
  }
  case quadlaxisym:{
    Lqat->transq_nodval(eid, nodval, nmq);
    break;
  }
  case ifacequadel:
    Ifcquadt->transq_nodval(eid, nodval, nmq);
    break;
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}



/**
  Function computes transport (non-mechanical) quantities in nodes of element.

  @param eid - element id
  @param nodval - %vector of nodal values of all required quantities, i.e., 
                  nodal value of i-th quantity in j-th node is given by nodval[i*ncnv+j] where ncnv 
                  is the number of calculated nodes on eid-th element.
  @param ncne - number of computed nodes on element (only first ncne of nodes is calculated)
  @param nq - number of required transport quantities
  @param qt - array of types of required transport quantities
   
  @return The function does not return anything.
 
  Created by Tomas Koudelka, 9.12.2013
*/
void elem_transq_nodval_comp (long eid,vector &nodval, long ncne, long nq, nonmechquant *qt)
{
  elemtypet te;

  te = Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->transq_nodval_comp (eid, nodval, ncne, nq, qt);
    break;
  }
  case quadlint:{
    Lqt->transq_nodval_comp (eid, nodval, ncne, nq, qt);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function returns integral over a single finite element
   variable is stored in nodes
   
   @param eid - number of element
   @param nodval - array of nodal values
   
   14.3.2013, JK
*/
double elem_total_integral (long eid,vector &nodval)
{
  double value;
  elemtypet te;
  
  //  type of element
  te = Tt->give_elem_type (eid);
  
  switch (te){
  case barlint:{      value = Lbt->total_integral (eid,nodval);   break;}
  case barlintax:{    value = Lbat->total_integral (eid,nodval);  break;}
  case barquadt:{     value = Qbt->total_integral (eid,nodval);   break;}
  case barquadtax:{   value = Qbat->total_integral (eid,nodval);  break;}
  case quadlint:{     value = Lqt->total_integral (eid,nodval);   break;}
  case quadquadt:{    value = Qqt->total_integral (eid,nodval);   break;}
  case quadquadtax:{  value = Qqat->total_integral (eid,nodval);  break;}
  case lineartett:{   value = Ltett->total_integral (eid,nodval); break;}
  case linearhext:{   value = Lht->total_integral (eid,nodval);   break;}
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return value;
}

/**
   function returns integral over a single finite element
   variable is stored in integration points
   
   @param eid - number of element
   @param varid - id of variable required
   
   3. 10. 2013, JK
*/
double elem_total_integral_ip (long eid,long varid)
{
  double value;
  elemtypet te;
  
  //  type of element
  te = Tt->give_elem_type (eid);
  
  switch (te){
  case barlint:{      value = Lbt->total_integral_ip (eid,varid);   break;}
    //case barlintax:{    value = Lbat->total_integral (eid,nodval);  break;}
    //case barquadt:{     value = Qbt->total_integral (eid,nodval);   break;}
    //case barquadtax:{   value = Qbat->total_integral (eid,nodval);  break;}
    //case quadlint:{     value = Lqt->total_integral (eid,nodval);   break;}
    //case quadquadt:{    value = Qqt->total_integral (eid,nodval);   break;}
    //case quadquadtax:{  value = Qqat->total_integral (eid,nodval);  break;}
    //case lineartett:{   value = Ltett->total_integral (eid,nodval); break;}
    //case linearhext:{   value = Lht->total_integral (eid,nodval);   break;}
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }//  end of the switch (te)
  
  return value;
}

